@@ -5,16 +5,21 @@
 
 from __future__ import print_function, unicode_literals
 
+import re
+import sys
+from os.path import join, normpath
+
+from .biffh import (
+    XL_CELL_BLANK, XL_CELL_BOOLEAN, XL_CELL_ERROR, XL_CELL_TEXT, XLRDError,
+    error_text_from_code,
+)
+from .book import Book, Name
+from .formatting import XF, Format, is_date_format_string
+from .sheet import Sheet
+from .timemachine import *
+
 DEBUG = 0
 
-from os.path import normpath, join
-import sys
-import re
-from .timemachine import *
-from .book import Book, Name
-from .biffh import error_text_from_code, XLRDError, XL_CELL_BLANK, XL_CELL_TEXT, XL_CELL_BOOLEAN, XL_CELL_ERROR
-from .formatting import is_date_format_string, Format, XF
-from .sheet import Sheet
 
 DLF = sys.stdout # Default Log File
 
@@ -28,20 +33,22 @@ def ensure_elementtree_imported(verbosity, logfile):
         return
     if "IronPython" in sys.version:
         import xml.etree.ElementTree as ET
-        #### 2.7.2.1: fails later with 
+        #### 2.7.2.1: fails later with
         #### NotImplementedError: iterparse is not supported on IronPython. (CP #31923)
     else:
-        try: import xml.etree.cElementTree as ET
+        try: import defusedxml.cElementTree as ET
         except ImportError:
-            try: import cElementTree as ET
+            try: import xml.etree.cElementTree as ET
             except ImportError:
-                try: import lxml.etree as ET
+                try: import cElementTree as ET
                 except ImportError:
-                    try: import xml.etree.ElementTree as ET
+                    try: import lxml.etree as ET
                     except ImportError:
-                        try: import elementtree.ElementTree as ET
+                        try: import xml.etree.ElementTree as ET
                         except ImportError:
-                            raise Exception("Failed to import an ElementTree implementation")
+                            try: import elementtree.ElementTree as ET
+                            except ImportError:
+                                raise Exception("Failed to import an ElementTree implementation")
     if hasattr(ET, 'iterparse'):
         _dummy_stream = BYTES_IO(b'')
         try:
@@ -49,15 +56,15 @@ def ensure_elementtree_imported(verbosity, logfile):
             ET_has_iterparse = True
         except NotImplementedError:
             pass
-    Element_has_iter = hasattr(ET.ElementTree, 'iter')
+    Element_has_iter = hasattr(ET, 'ElementTree') and hasattr(ET.ElementTree, 'iter')
     if verbosity:
         etree_version = repr([
             (item, getattr(ET, item))
             for item in ET.__dict__.keys()
             if item.lower().replace('_', '') == 'version'
-            ])
+        ])
         print(ET.__file__, ET.__name__, etree_version, ET_has_iterparse, file=logfile)
-        
+
 def split_tag(tag):
     pos = tag.rfind('}') + 1
     if pos >= 2:
@@ -127,9 +134,8 @@ F_TAG = U_SSML12 + 'f' # cell child: formula
 IS_TAG = U_SSML12 + 'is' # cell child: inline string
 
 def unescape(s,
-    subber=re.compile(r'_x[0-9A-Fa-f]{4,4}_', re.UNICODE).sub,
-    repl=lambda mobj: unichr(int(mobj.group(0)[2:6], 16)),
-    ):
+             subber=re.compile(r'_x[0-9A-Fa-f]{4,4}_', re.UNICODE).sub,
+             repl=lambda mobj: unichr(int(mobj.group(0)[2:6], 16))):
     if "_" in s:
         return subber(repl, s)
     return s
@@ -220,7 +226,7 @@ _defined_name_attribute_map = (
     ("",                    "option_flags", 0,               ),
     ("",                    "result",       None,            ),
     ("",                    "stack",        None,            ),
-    )
+)
 
 def make_name_access_maps(bk):
     name_and_scope_map = {} # (name.lower(), scope): Name_object
@@ -295,7 +301,7 @@ class X12Book(X12General):
         U_DC+"creator": ("creator", cnv_ST_Xstring),
         U_DCTERMS+"modified": ("modified", cnv_ST_Xstring),
         U_DCTERMS+"created": ("created", cnv_ST_Xstring),
-        }
+    }
 
     def process_coreprops(self, stream):
         if self.verbosity >= 2:
@@ -386,8 +392,8 @@ class X12Book(X12General):
             None: 0,
             'visible': 0,
             'hidden': 1,
-            'veryHidden': 2
-            }
+            'veryHidden': 2,
+        }
         bk._sheet_visibility.append(visibility_map[state])
         sheet = Sheet(bk, position=None, name=name, number=sheetx)
         sheet.utter_max_rows = X12_MAX_ROWS
@@ -409,7 +415,7 @@ class X12Book(X12General):
         'definedNames':  do_defined_names,
         'workbookPr':   do_workbookpr,
         'sheet':        do_sheet,
-        }
+    }
     augment_keys(tag2meth, U_SSML12)
 
 class X12SST(X12General):
@@ -422,7 +428,7 @@ class X12SST(X12General):
             self.process_stream = self.process_stream_iterparse
         else:
             self.process_stream = self.process_stream_findall
-            
+
     def process_stream_iterparse(self, stream, heading=None):
         if self.verbosity >= 2 and heading is not None:
             fprintf(self.logfile, "\n=== %s ===\n", heading)
@@ -436,7 +442,7 @@ class X12SST(X12General):
                 fprintf(self.logfile, "element #%d\n", elemno)
                 self.dump_elem(elem)
             result = get_text_from_si_or_is(self, elem)
-            sst.append(result)                
+            sst.append(result)
             elem.clear() # destroy all child elements
         if self.verbosity >= 2:
             self.dumpout('Entries in SST: %d', len(sst))
@@ -506,10 +512,7 @@ class X12Styles(X12General):
         is_date = self.fmt_is_date.get(numFmtId, 0)
         self.bk._xf_index_to_xl_type_map[xfx] = is_date + 2
         if self.verbosity >= 3:
-            self.dumpout(
-                'xfx=%d numFmtId=%d',
-                xfx, numFmtId,
-                )
+            self.dumpout('xfx=%d numFmtId=%d', xfx, numFmtId)
             self.dumpout(repr(self.bk._xf_index_to_xl_type_map))
 
     tag2meth = {
@@ -517,7 +520,7 @@ class X12Styles(X12General):
         'cellXfs':      do_cellxfs,
         'numFmt':       do_numfmt,
         'xf':           do_xf,
-        }
+    }
     augment_keys(tag2meth, U_SSML12)
 
 class X12Sheet(X12General):
@@ -540,7 +543,6 @@ class X12Sheet(X12General):
     def own_process_stream(self, stream, heading=None):
         if self.verbosity >= 2 and heading is not None:
             fprintf(self.logfile, "\n=== %s ===\n", heading)
-        getmethod = self.tag2meth.get
         row_tag = U_SSML12 + "row"
         self_do_row = self.do_row
         for event, elem in ET.iterparse(stream):
@@ -606,17 +608,22 @@ class X12Sheet(X12General):
         # The ref attribute should be a cell range like "B1:D5".
         ref = elem.get('ref')
         if ref:
-            first_cell_ref, last_cell_ref = ref.split(':')
+            try:
+                first_cell_ref, last_cell_ref = ref.split(':')
+            except ValueError:
+                # encountered a single cell merge, e.g. "B3"
+                first_cell_ref = ref
+                last_cell_ref = ref
             first_rowx, first_colx = cell_name_to_rowx_colx(first_cell_ref)
             last_rowx, last_colx = cell_name_to_rowx_colx(last_cell_ref)
             self.merged_cells.append((first_rowx, last_rowx + 1,
                                       first_colx, last_colx + 1))
 
     def do_row(self, row_elem):
-    
+
         def bad_child_tag(child_tag):
-             raise Exception('cell type %s has unexpected child <%s> at rowx=%r colx=%r' % (cell_type, child_tag, rowx, colx))
- 
+            raise Exception('cell type %s has unexpected child <%s> at rowx=%r colx=%r' % (cell_type, child_tag, rowx, colx))
+
         row_number = row_elem.get('r')
         if row_number is None: # Yes, it's optional.
             self.rowx += 1
@@ -665,7 +672,6 @@ class X12Sheet(X12General):
             xf_index = int(cell_elem.get('s', '0'))
             cell_type = cell_elem.get('t', 'n')
             tvalue = None
-            formula = None
             if cell_type == 'n':
                 # n = number. Most frequent type.
                 # <v> child contains plain text which can go straight into float()
@@ -675,7 +681,8 @@ class X12Sheet(X12General):
                     if child_tag == V_TAG:
                         tvalue = child.text
                     elif child_tag == F_TAG:
-                        formula = cooked_text(self, child)
+                        # formula
+                        pass
                     else:
                         raise Exception('unexpected tag %r' % child_tag)
                 if not tvalue:
@@ -692,7 +699,7 @@ class X12Sheet(X12General):
                         tvalue = child.text
                     elif child_tag == F_TAG:
                         # formula not expected here, but gnumeric does it.
-                        formula = child.text
+                        pass
                     else:
                         bad_child_tag(child_tag)
                 if not tvalue:
@@ -711,7 +718,8 @@ class X12Sheet(X12General):
                     if child_tag == V_TAG:
                         tvalue = cooked_text(self, child)
                     elif child_tag == F_TAG:
-                        formula = cooked_text(self, child)
+                        # formula
+                        pass
                     else:
                         bad_child_tag(child_tag)
                 # assert tvalue is not None and formula is not None
@@ -720,26 +728,27 @@ class X12Sheet(X12General):
             elif cell_type == "b":
                 # b = boolean
                 # <v> child contains "0" or "1"
-                # Maybe the data should be converted with cnv_xsd_boolean;
-                # ECMA standard is silent; Excel 2007 writes 0 or 1
                 for child in cell_elem:
                     child_tag = child.tag
                     if child_tag == V_TAG:
                         tvalue = child.text
                     elif child_tag == F_TAG:
-                        formula = cooked_text(self, child)
+                        # formula
+                        pass
                     else:
                         bad_child_tag(child_tag)
-                self.sheet.put_cell(rowx, colx, XL_CELL_BOOLEAN, int(tvalue), xf_index)
+                self.sheet.put_cell(rowx, colx, XL_CELL_BOOLEAN, cnv_xsd_boolean(tvalue), xf_index)
             elif cell_type == "e":
                 # e = error
                 # <v> child contains e.g. "#REF!"
+                tvalue = '#N/A'
                 for child in cell_elem:
                     child_tag = child.tag
                     if child_tag == V_TAG:
                         tvalue = child.text
                     elif child_tag == F_TAG:
-                        formula = cooked_text(self, child)
+                        # formula
+                        pass
                     else:
                         bad_child_tag(child_tag)
                 value = error_code_from_text[tvalue]
@@ -755,7 +764,8 @@ class X12Sheet(X12General):
                     elif child_tag == V_TAG:
                         tvalue = child.text
                     elif child_tag == F_TAG:
-                        formula = child.text
+                        # formula
+                        pass
                     else:
                         bad_child_tag(child_tag)
                 if not tvalue:
@@ -768,19 +778,17 @@ class X12Sheet(X12General):
 
     tag2meth = {
         'row':          do_row,
-        }
+    }
     augment_keys(tag2meth, U_SSML12)
 
-def open_workbook_2007_xml(
-    zf,
-    component_names,
-    logfile=sys.stdout,
-    verbosity=0,
-    use_mmap=0,
-    formatting_info=0,
-    on_demand=0,
-    ragged_rows=0,
-    ):
+def open_workbook_2007_xml(zf,
+                           component_names,
+                           logfile=sys.stdout,
+                           verbosity=0,
+                           use_mmap=0,
+                           formatting_info=0,
+                           on_demand=0,
+                           ragged_rows=0):
     ensure_elementtree_imported(verbosity, logfile)
     bk = Book()
     bk.logfile = logfile
